package com.yj.web;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.lang.reflect.Method;

/**
 * @author yj
 * @create 2020-08-23 16:51
 *
 * BaseServlet类是一个抽象的Servlet基类，用于简化Servlet的开发。
 * 它通过反射机制动态调用子类中的方法，避免了在每个Servlet中重复编写doGet和doPost方法。
 */
public abstract class BaseServlet extends HttpServlet {

    /**
     * 处理GET请求，直接调用doPost方法
     * @param req HttpServletRequest对象
     * @param resp HttpServletResponse对象
     * @throws ServletException
     * @throws IOException
     */
    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        doPost(req, resp);
    }

    /**
     * 处理POST请求，通过反射机制动态调用子类中的方法
     * @param req HttpServletRequest对象
     * @param resp HttpServletResponse对象
     * @throws ServletException
     * @throws IOException
     */
    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        // 解决POST请求中文乱码问题
        req.setCharacterEncoding("utf-8");

        // 解决响应的中文乱码问题
        resp.setContentType("text/html;charset=UTF-8");

        // 获取请求参数action，该参数用于指定要调用的方法名
        String action = req.getParameter("action");

        try {
            // 通过反射获取当前类中指定名称的方法
            Method method = this.getClass().getDeclaredMethod(action, HttpServletRequest.class, HttpServletResponse.class);

            // 调用方法，传入请求和响应对象
            method.invoke(this, req, resp);
        } catch (Exception e) {
            // 捕获异常并打印堆栈信息
            e.printStackTrace();

            // 将异常包装为运行时异常抛出，便于上层处理
            throw new RuntimeException(e);
        }
    }
}